---- Packer

--- Init

-- Ensure Packer.nvim is installed and load it
local ensure_packer = function()
  local fn = vim.fn
  local install_path = fn.stdpath('data')..'/site/pack/packer/start/packer.nvim'
  
  -- Check if Packer.nvim is already installed
  if fn.empty(fn.glob(install_path)) > 0 then
    -- If not installed, clone it from GitHub
    fn.system({'git', 'clone', '--depth', '1', 'https://github.com/wbthomason/packer.nvim', install_path})
    -- Load Packer.nvim
    vim.cmd [[packadd packer.nvim]]
    return true
  end
  return false
end

local packer_bootstrap = ensure_packer()

--- Plugins

-- Check if Packer.nvim was bootstrapped successfully and sync plugins
if packer_bootstrap then
  require('packer').sync()
end

-- Define and configure plugins using Packer.nvim
require('packer').startup(function(use)
use 'wbthomason/packer.nvim'
use 'junegunn/goyo.vim'                        -- Zen mode for distraction-free writing
use 'vehementham/onedark.nvim'                 -- One Dark color scheme
use { "catppuccin/nvim", as = "catppuccin" }   -- Catppuccin color scheme
use 'numToStr/Comment.nvim'                    -- Commenting utility
use {
  'kyazdani42/nvim-tree.lua',                  -- Filesystem navigation
  requires = 'kyazdani42/nvim-web-devicons'    -- Filesystem icons
}
use 'mhinz/vim-startify'                       -- Start screen
use 'DanilaMihailov/beacon.nvim'               -- Cursor jump indicator
use 'itchyny/lightline.vim'                    -- Lightline
use 'majutsushi/tagbar'                        -- Code structure
use 'Yggdroot/indentLine'                      -- See indentation
use 'tpope/vim-fugitive'                       -- Git integration
use 'junegunn/gv.vim'                          -- Commit history
use 'windwp/nvim-autopairs'                    -- Auto close brackets, etc. 
use 'rktjmp/lush.nvim'                         -- Assisted colorscheme creation 
use {
    'nvim-treesitter/nvim-treesitter',         -- Treesitter configurations and abstraction layer
    run = function()
    local ts_update = require('nvim-treesitter.install').update({ with_sync = true })
    ts_update()
end,
}
use {
    "nvim-neorg/neorg",
    run = ":Neorg sync-parsers",  -- This is the important bit!
    config = function()
        require("neorg").setup {
                                  -- configuration here
        }  
    end,
}

-- Configure the package_root for Neovim packages
config = {
  package_root = vim.fn.stdpath('config') .. '/site/pack'
}
end)



---- Lightline

-- Load Lightline
vim.cmd [[set laststatus=2]]
vim.cmd [[set noshowmode]]



---- Comments

-- Enable Comment.nvim plugin and set up its configuration
require('Comment').setup()



---- Nvim-Tree configuration

-- Enable Nvim-Tree
require('nvim-tree').setup{}



---- Apperence

--- Main

-- Onedark 

--[=[
-- Set colorscheme to 'onedark'
vim.cmd [[let g:onedark_termcolors=16]] -- Enable terminal colors
vim.cmd [[colorscheme onedark]]

-- Set transparent background
vim.api.nvim_command('highlight Normal guibg=NONE ctermbg=NONE')
--]=]

-- Catppuccin

require("catppuccin").setup({
    flavour = "macchiato", -- latte, frappe, macchiato, mocha
    background = { -- :h background
        light = "latte",
        dark = "macchiato",
    },
    transparent_background = true, -- disables setting the background color.
    show_end_of_buffer = false, -- shows the '~' characters after the end of buffers
    term_colors = true, -- sets terminal colors (e.g. `g:terminal_color_0`)
    dim_inactive = {
        enabled = false, -- dims the background color of inactive window
        shade = "dark",
        percentage = 0.15, -- percentage of the shade to apply to the inactive window
    },
    no_italic = false, -- Force no italic
    no_bold = false, -- Force no bold
    no_underline = false, -- Force no underline
    styles = { -- Handles the styles of general hi groups (see `:h highlight-args`):
        comments = { "italic" }, -- Change the style of comments
        conditionals = { "italic" },
        loops = {},
        functions = {},
        keywords = {},
        strings = {},
        variables = {},
        numbers = {},
        booleans = {},
        properties = {},
        types = {},
        operators = {},
    },
    color_overrides = {},
    custom_highlights = {},
    integrations = {
        cmp = true,
        gitsigns = true,
        nvimtree = true,
        treesitter = true,
        notify = false,
        mini = {
            enabled = true,
            indentscope_color = "",
        },
        -- For more plugins integrations please scroll down (https://github.com/catppuccin/nvim#integrations)
    },
})

-- setup must be called before loading
vim.cmd.colorscheme "catppuccin"

--- Lightline

require("catppuccin").setup({
    integrations = {
        nvimtree = true,
        treesitter = true,
    }
})

-- Set colorscheme for lightline
vim.g.lightline = {
  colorscheme = 'catppuccin',

-- Powerline Extra Symbols
  separator = { left = '', right = '' },
  subseparator = { left = '', right = '' },
}


---- Vim Commands

vim.cmd [[set clipboard+=unnamedplus]] -- Clipbord



---- Key mappings

local map = vim.api.nvim_set_keymap

-- Toggle Nvim-Tree with 'n' key
map('n', ',', ':NvimTreeToggle<CR>', {})

--- Insert Mode Bindings

-- Remap the key used to leave insert mode to 'jk'
map('i', 'jk', '', {})

-- Move right by a word in insert mode
vim.api.nvim_set_keymap('i', '<C-Right>', '<Esc>ea', { noremap = true, silent = true })

-- Move left by a word
vim.api.nvim_set_keymap('i', '<C-Left>', '<Esc>bi', { noremap = true, silent = true })

-- Map Ctrl + Delete to delete the word to the right
vim.api.nvim_set_keymap('i', '<C-Delete>', 'X<Esc>lbce', { noremap = true, silent = true })

-- Map Ctrl + Backspace to delete the word to the left
vim.api.nvim_set_keymap('i', '<C-Backspace>', '<C-w>', { noremap = true, silent = true })

--- Dvorak

-- Unmap d and D
vim.api.nvim_set_keymap('n', 'd', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'D', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'd', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'D', '<nop>', { noremap = true, silent = true })

-- Unmap n and N
vim.api.nvim_set_keymap('n', 'n', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'N', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'n', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'N', '<nop>', { noremap = true, silent = true })

-- Unmap HJKL
vim.api.nvim_set_keymap('n', 'h', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'j', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'k', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'l', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'h', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'j', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'k', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'l', '<nop>', { noremap = true, silent = true })

vim.api.nvim_set_keymap('n', 'H', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'J', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'K', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'L', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'H', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'J', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'K', '<nop>', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'L', '<nop>', { noremap = true, silent = true })

-- Remap lowercase d to k, and uppercase D to K
vim.api.nvim_set_keymap('n', 'k', 'd', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'K', 'D', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'k', 'd', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'K', 'D', { noremap = true, silent = true })

-- Remap lowercase n to l, and uppercase N to L
vim.api.nvim_set_keymap('n', 'l', 'n', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'L', 'N', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'l', 'n', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'L', 'N', { noremap = true, silent = true })


-- Remap DHTN to HJKL
vim.api.nvim_set_keymap('n', 'd', 'h', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'h', 'j', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 't', 'k', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'n', 'l', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'd', 'h', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'h', 'j', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 't', 'k', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'n', 'l', { noremap = true, silent = true })


vim.api.nvim_set_keymap('n', 'D', 'H', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'H', 'J', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'T', 'K', { noremap = true, silent = true })
vim.api.nvim_set_keymap('n', 'N', 'L', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'D', 'H', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'H', 'J', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'T', 'K', { noremap = true, silent = true })
vim.api.nvim_set_keymap('v', 'N', 'L', { noremap = true, silent = true })


-- Map "kk" to delete a line in normal mode
vim.api.nvim_set_keymap('n', 'kk', ':delete<CR>', { noremap = true, silent = true })



